@@ -26,7 +26,7 @@ from datacube.ui.click import config_option, environment_option
 from datacube.utils.uris import normalise_path
 from defusedxml import minidom
 
-from eodatasets3 import DatasetDoc, DatasetPrepare, serialise
+from eodatasets3 import DatasetDoc, DatasetPrepare, names, serialise
 from eodatasets3.properties import Eo3Interface
 from eodatasets3.ui import PathPath
 from eodatasets3.utils import pass_config
@@ -248,6 +248,8 @@ def prepare_and_write(
         doc = serialise.from_doc(
             p.written_dataset_doc, skip_validation=True, normalise_properties=False
         )
+        if not doc.locations:
+            doc.locations = [names.resolve_location(dataset_location)]
         return doc, metadata_path
 
 
@@ -551,6 +553,37 @@ def main(
 
     _LOG.info(f"{len(datasets)} paths(s) to process using {workers} worker(s))")
 
+    # Are we indexing on success?
+    index = None
+    if index_to_odc:
+        _LOG.info("Indexing new datasets")
+        if local_config:
+            _LOG.debug("Indexing to %s", local_config)
+        index = index_connect(local_config, application_name="s2-prepare")
+        products = {}
+
+        def on_success(dataset: DatasetDoc, dataset_path: Path):
+            """
+            Index the dataset
+            """
+            product_name = dataset.product.name
+            product = products.get(product_name)
+            if not product:
+                product = index.products.get_by_name(product_name)
+                if not product:
+                    raise ValueError(f"Product {product_name} not found in ODC index")
+                products[product_name] = product
+
+            index.datasets.add(
+                Dataset(product, serialise.to_doc(dataset), uris=dataset.locations)
+            )
+            _LOG.debug("Indexed dataset %s to %s", dataset.id, dataset_path)
+
+    else:
+
+        def on_success(dataset: DatasetDoc, dataset_path: Path):
+            """Nothing extra"""
+
     def files_in_path(input_path: Path):
         """
         Scan the input path for our key identifying files of a package.
@@ -673,36 +706,6 @@ def main(
     # (Multiprocessing makes debugging harder, so we prefer to make it optional)
     successes = 0
 
-    # Are we indexing on success?
-    index = None
-    if index_to_odc:
-        _LOG.info("Indexing new datasets")
-        if local_config:
-            _LOG.debug("Indexing to %s", local_config)
-        index = index_connect(local_config, application_name="s2-prepare")
-        products = {}
-
-        def on_success(dataset: DatasetDoc, dataset_path: Path):
-            """
-            Index the dataset
-            """
-            product_name = dataset.product.name
-            product = products.get(product_name)
-            if not product:
-                product = index.products.get_by_name(product_name)
-                if not product:
-                    raise ValueError(f"Product {product_name} not found in ODC index")
-                products[product_name] = product
-
-            index.datasets.add(Dataset(product, serialise.to_doc(dataset)))
-            _LOG.debug("Wrote and indexed dataset %s to %s", dataset.id, dataset_path)
-
-    else:
-
-        def on_success(dataset: DatasetDoc, dataset_path: Path):
-            """Nothing extra"""
-            _LOG.debug("Wrote dataset %s to %s", dataset.id, dataset_path)
-
     try:
         if workers == 1 or dry_run:
             for job in find_jobs():
@@ -720,10 +723,11 @@ def main(
                             job.producer,
                             embed_location=job.embed_location,
                         )
+                        _LOG.debug("Wrote dataset %s to %s", dataset.id, path)
                         on_success(dataset, path)
                     successes += 1
                 except Exception:
-                    _LOG.exception("Failed to write dataset: %s", job)
+                    _LOG.exception("Failed to complete dataset: %s", job)
                     errors += 1
         else:
             with Pool(processes=workers) as pool:
@@ -733,6 +737,7 @@ def main(
                         errors += 1
                     else:
                         dataset, path = res
+                        _LOG.debug("Wrote dataset %s to %s", dataset.id, path)
                         on_success(dataset, path)
                         successes += 1
                 pool.close()
